searchNodes=[{"type":"module","title":"NextPipe","doc":"Use import NextPipe to make your pipelines a bit more flexible. NextPipe allows the chaining of functions with control through the idiomatic {:ok, _} and {:error, _} tuples. In the case of a function returning a value matching {:error, _} , the pipeline is short-circuited. value |&gt; next ( fn ... ) |&gt; next ( fn ... ) |&gt; try_next ( fn ... , fn ... ) |&gt; on_error ( fn ... ) |&gt; always ( fn ... ) NextPipe doesn't use macros or overridden operators. Like Kernel.then/2 , the functions in NextPipe work with function arguments and idiomatic tuples, {:ok, _} and {:error, _} . Use next/2 to conditionally execute its function argument based on the value of the first argument. If the first argument matches {:ok, _} , the function passed as the second argument will be called with the second element of the tuple passed in the first argument. If the first argument matches {:error, _} , the function will not be called and the same tuple will be returned by next/2 . Otherwise (like at the beginning of a pipeline), the function will be called with value . try_next/3 works like next/2 but rescues exceptions. It accepts a third optional argument, which is the function to be called in case an exception is rescued. Use always/2 to always call the function argument, but with the full argument value, not just the second element of the tuple. Use on_error/2 to respond to an error tuple. As an alternative to with The with special form is often use to conditionally call functions if prior functions are successful: with { :ok , value } &lt;- fn1 ( arg1 ) , { :ok , value } &lt;- fn2 ( value , arg2 ) do fn3 ( value ) else { :error , error } -&gt; error_fn ( error ) end With NextPipe : arg1 |&gt; next ( &amp; fn1 / 1 ) |&gt; next ( &amp; fn2 ( &amp;1 , arg2 ) ) |&gt; next ( &amp; fn3 / 1 ) |&gt; on_error ( &amp; error_fn / 1 ) In the case of with , if a function result does not match with the left side of the &lt;- operator, the subsequent clauses are skipped. If there is an else in the with it is used to handle the mismatch. When using next/2 , if the argument matches {:error, _} then the function argument is not called and the first argument is returned unchanged. In this way, all next/2 calls are effectively skipped in the pipeline. If one of the functions may raise an exception, more boilerplate code is eliminated. Compare using with : try do with { :ok , value } &lt;- fn1 ( arg1 ) , { :ok , value } &lt;- fn2 ( value , arg2 ) do fn3 ( value ) end rescue exception -&gt; { :error , exception } end to the equivalent behavior using NextPipe : arg1 |&gt; try_next ( &amp; fn1 ( &amp;1 ) ) |&gt; try_next ( &amp; fn2 ( &amp;1 , arg2 ) ) |&gt; try_next ( &amp; fn3 ( &amp;1 ) ) As an alternative to Ecto.Multi Transaction control with Ecto.Multi is quite powerful and flexible. It can, however, be a bit cumbersome for simpler situations. And then Repo.transaction/2 with a simple function requires some boilerplate code for rescuing any exeptions if passing those up is undesirable. NextPipe may clean those cases up a bit. Compare this use of Repo.transaction/2 : def something ( arg1 , arg2 ) do try do Repo . transaction ( fn repo -&gt; arg1 |&gt; fn1 ( arg2 ) |&gt; fn2 ( ) end ) rescue exception -&gt; { :error , exception } end end And then using NextPipe : def something ( arg1 , arg2 ) do Repo . transaction ( fn repo -&gt; arg1 |&gt; try_next ( &amp; fn1 ( &amp;1 , arg2 ) ) |&gt; try_next ( &amp; fn2 ( &amp;1 ) ) |&gt; on_error ( fn error -&gt; repo . rollback ( error ) ) end ) end Functions with multiple arguments The function passed to next/2 et al accepts a single argument. If multiple arguments are required, return a new function with those arguments bound. As an example, consider the following traditional Elixir pipeline: def something ( arg1 , arg2 ) do arg1 |&gt; fn1 ( arg2 ) |&gt; fn2 ( ) end The analogous pipeline using next/2 might be: def something ( arg1 , arg2 ) do arg1 |&gt; next ( &amp; fn1 ( &amp;1 , arg2 ) ) |&gt; next ( &amp; fn2 ( &amp;1 ) ) end","ref":"NextPipe.html"},{"type":"function","title":"NextPipe.always/2","doc":"Always call always_fn with the first argument. Unlike next , the always_fn receives the full tuple passed as the first argument to always/2 , i.e., {:ok, value} or {:error, value} . always_fn should return a value matching either {:ok, _} or {:error, _} . This is basically the same as then/2 with a more strict type spec.","ref":"NextPipe.html#always/2"},{"type":"function","title":"NextPipe.next/2","doc":"Conditionally call the next function. If the first argument matches {:ok, value} , next_fn is called with value . If the first argument matches {:error, _} , the call to next_fn will be skipped and the function returns the first argument unchanged. Otherwise, the function is called with first argument unchanged. This supports the use of next/2 at the beginning of a pipeline. next/2 returns the value returned by next_fn . next_fn typically returns {:ok, value} if it was successful. Otherwise, return {:error, value} . It may also make sense for next_fn to return a value of a different shape, like in the last step of a pipeline.","ref":"NextPipe.html#next/2"},{"type":"function","title":"NextPipe.next_while/2","doc":"Process the enumerable against the function while the function returns {:ok, _} tuples. If all items are successful, the result is an {:ok, items} tuple, where items is the list of values returned as the seccond item in the {:ok, item} tuple from function . If any call to function is unsuccessful, the loop is halted and the return value is {:error, {error, items}} , where error is the error returned from the unsuccessful function call and items is the list of items processed successfully.","ref":"NextPipe.html#next_while/2"},{"type":"function","title":"NextPipe.ok/1","doc":"Wrap the argument in an {:ok, _} tuple, if necessary. This is useful for returning an {:ok, _} tuple at the end of a pipeline instead of, for example, |&gt; then(&amp;{:ok, &amp;1}) {:ok, value} - return unchanged {:error, value} - return unchanged value - return {:ok, value}","ref":"NextPipe.html#ok/1"},{"type":"function","title":"NextPipe.ok/2","doc":"An alias for next/2 .","ref":"NextPipe.html#ok/2"},{"type":"function","title":"NextPipe.on_error/2","doc":"The inverse of next/2 , if the first argument matches {:error, value} , call error_fn with value . Otherwise, return the first argument.","ref":"NextPipe.html#on_error/2"},{"type":"function","title":"NextPipe.try_next/3","doc":"Conditionally call the next function and provide an outlet for handling exceptions when next_fn is called. This function generally behaves the same as next/2 unless an exception is raised in next_fn . In that case, the exception is rescued and rescue_fn is called with two arguments: the second element of the {:ok, value} tuple and the raised exception. The value returned from try_next/3 is then the value returned from rescue_fn . rescue_fn should return a value matching {:ok, _} or {:error, _} . The default implementation of rescue_fn simply returns an {:error, exception} tuple, where exception is the exception raised by next_fn .","ref":"NextPipe.html#try_next/3"},{"type":"extras","title":"NextPipe","doc":"Mix . install ( [ { :next_pipe , &quot;~&gt; 0.1&quot; } ] ) :ok","ref":"next_pipe.html"},{"type":"extras","title":"NextPipe - Import","doc":"import NextPipe NextPipe","ref":"next_pipe.html#import"},{"type":"extras","title":"NextPipe - Next","doc":"next/2 : evaulate the function argument if the first argument matches {:ok, _} :zero |&gt; next ( fn :zero -&gt; { :ok , :one } end ) |&gt; next ( fn :one -&gt; { :ok , :two } end ) |&gt; next ( fn :two -&gt; { :ok , :three } end ) # on_error is skipped because there&#39;s no error |&gt; on_error ( fn :four -&gt; { :ok , :five } end ) |&gt; dbg ( ) [ notebooks / next_pipe . livemd #cell:7: (file)] :zero #=&gt; :zero |&gt; next ( fn :zero -&gt; { :ok , :one } end ) #=&gt; {:ok, :one} |&gt; next ( fn :one -&gt; { :ok , :two } end ) #=&gt; {:ok, :two} |&gt; next ( fn :two -&gt; { :ok , :three } end ) #=&gt; {:ok, :three} |&gt; on_error ( fn :four -&gt; { :ok , :five } end ) #=&gt; {:ok, :three} { :ok , :three } :zero |&gt; next ( fn :zero -&gt; { :ok , :one } end ) |&gt; next ( fn :one -&gt; { :ok , :two } end ) |&gt; next ( fn :two -&gt; { :error , :three } end ) # the function passed below is never evaluated because there&#39;s an error |&gt; next ( fn :nope -&gt; { :ok , :four } end ) # the function below is evaulated because there is an error |&gt; on_error ( fn :three -&gt; { :ok , :five } end ) |&gt; dbg ( ) [ notebooks / next_pipe . livemd #cell:9: (file)] :zero #=&gt; :zero |&gt; next ( fn :zero -&gt; { :ok , :one } end ) #=&gt; {:ok, :one} |&gt; next ( fn :one -&gt; { :ok , :two } end ) #=&gt; {:ok, :two} |&gt; next ( fn :two -&gt; { :error , :three } end ) #=&gt; {:error, :three} |&gt; next ( fn :nope -&gt; { :ok , :four } end ) #=&gt; {:error, :three} |&gt; on_error ( fn :three -&gt; { :ok , :five } end ) #=&gt; {:ok, :five} { :ok , :five }","ref":"next_pipe.html#next"},{"type":"extras","title":"NextPipe - Try Next","doc":"try_next/3 : evaulate the function argument if the first argument matches {:ok, _} and provide an outlet for exceptions :zero |&gt; try_next ( fn :zero -&gt; { :ok , :one } end ) |&gt; try_next ( fn :one -&gt; { :ok , :two } end ) |&gt; try_next ( fn :two -&gt; raise &quot;three&quot; end ) # The function below is skipped because there&#39;s an error with the caught exception |&gt; try_next ( fn :nope -&gt; { :ok , :four } end ) |&gt; on_error ( fn % RuntimeError { message : &quot;three&quot; } -&gt; { :ok , :five } end ) |&gt; dbg ( ) [ notebooks / next_pipe . livemd #cell:8: (file)] :zero #=&gt; :zero |&gt; try_next ( fn :zero -&gt; { :ok , :one } end ) #=&gt; {:ok, :one} |&gt; try_next ( fn :one -&gt; { :ok , :two } end ) #=&gt; {:ok, :two} |&gt; try_next ( fn :two -&gt; raise &quot;three&quot; end ) #=&gt; {:error, %RuntimeError{message: &quot;three&quot;}} |&gt; try_next ( fn :nope -&gt; { :ok , :four } end ) #=&gt; {:error, %RuntimeError{message: &quot;three&quot;}} |&gt; on_error ( fn % RuntimeError { message : &quot;three&quot; } -&gt; { :ok , :five } end ) #=&gt; {:ok, :five} { :ok , :five } :zero |&gt; try_next ( fn :zero -&gt; { :ok , :one } end ) |&gt; try_next ( fn :one -&gt; { :ok , :two } end ) # Override the rescue function and return an `{:ok, _}` tuple |&gt; try_next ( fn :two -&gt; raise &quot;three&quot; end , fn :two , % RuntimeError { message : &quot;three&quot; } -&gt; { :ok , :three } end ) |&gt; try_next ( fn :three -&gt; { :ok , :four } end ) # The function below is skipped because the exception was rescued |&gt; on_error ( fn % RuntimeError { message : &quot;three&quot; } -&gt; { :ok , :five } end ) |&gt; dbg ( ) [ notebooks / next_pipe . livemd #cell:11: (file)] :zero #=&gt; :zero |&gt; try_next ( fn :zero -&gt; { :ok , :one } end ) #=&gt; {:ok, :one} |&gt; try_next ( fn :one -&gt; { :ok , :two } end ) #=&gt; {:ok, :two} |&gt; try_next ( fn :two -&gt; raise &quot;three&quot; end , fn :two , % RuntimeError { message : &quot;three&quot; } -&gt; { :ok , :three } end ) #=&gt; {:ok, :three} |&gt; try_next ( fn :three -&gt; { :ok , :four } end ) #=&gt; {:ok, :four} |&gt; on_error ( fn % RuntimeError { message : &quot;three&quot; } -&gt; { :ok , :five } end ) #=&gt; {:ok, :four} { :ok , :four }","ref":"next_pipe.html#try-next"},{"type":"extras","title":"NextPipe - OK","doc":"ok/2 : an alias for next/2 :zero |&gt; ok ( fn :zero -&gt; { :ok , :one } end ) |&gt; ok ( fn :one -&gt; { :ok , :two } end ) |&gt; ok ( fn :two -&gt; { :error , :three } end ) |&gt; ok ( fn :nope -&gt; { :ok , :four } end ) |&gt; on_error ( fn :three -&gt; { :ok , :five } end ) |&gt; dbg ( ) [ notebooks / next_pipe . livemd #cell:7: (file)] :zero #=&gt; :zero |&gt; ok ( fn :zero -&gt; { :ok , :one } end ) #=&gt; {:ok, :one} |&gt; ok ( fn :one -&gt; { :ok , :two } end ) #=&gt; {:ok, :two} |&gt; ok ( fn :two -&gt; { :error , :three } end ) #=&gt; {:error, :three} |&gt; ok ( fn :nope -&gt; { :ok , :four } end ) #=&gt; {:error, :three} |&gt; on_error ( fn :three -&gt; { :ok , :five } end ) #=&gt; {:ok, :five} { :ok , :five }","ref":"next_pipe.html#ok"}]